/*
 * Copyright 2016 The Closure Compiler Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

goog.module('jscomp.runtime_tests.polyfill_tests.array_entries_test');
goog.setTestOnly();

const testSuite = goog.require('goog.testing.testSuite');
const userAgent = goog.require('goog.userAgent');
const testing = goog.require('jscomp.runtime_tests.polyfill_tests.testing');

const assertIteratorContents = testing.assertIteratorContents;
const noCheck = testing.noCheck;

testSuite({
  shouldRunTests() {
    // Disable tests for IE8 and below.
    return !userAgent.IE || userAgent.isVersionOrHigher(9);
  },

  testEntries() {
    assertIteratorContents(['x', 'c'].entries(), [0, 'x'], [1, 'c']);

    const arr = new Array();
    arr[3] = 42;
    assertIteratorContents(
        arr.entries(), [0, void 0], [1, void 0], [2, void 0], [3, 42]);

    assertIteratorContents(
        [].entries.call(noCheck({length: 3, 1: 'c'})),
        [0, void 0], [1, 'c'], [2, void 0]);

    assertIteratorContents(
        Array.prototype.entries.call(noCheck('xc')),
        [0, 'x'], [1, 'c']);
  },
});
